"""
**************************************

NeuroGenome

Created in 20.11.2021
by Aiyyskhan Alekseev 

https://github.com/Aiyyskhan
aiyyskhan@gmail.com

License: MIT

**************************************
"""
__version__ = "0.1.4"

import neurogenome.genome as genome
import neurogenome.builders as builders
import neurogenome.evolution as evolution
import neurogenome.macromutations as macromutations
import neurogenome.file_handler as file_handler
